'use strict';

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _nesthydrationjs = require('nesthydrationjs');

var _queryASTToSqlAST = require('./queryASTToSqlAST');

var _queryASTToSqlAST2 = _interopRequireDefault(_queryASTToSqlAST);

var _stringifySqlAST = require('./stringifySqlAST');

var _stringifySqlAST2 = _interopRequireDefault(_stringifySqlAST);

var _makeNestHydrationSpec = require('./makeNestHydrationSpec');

var _makeNestHydrationSpec2 = _interopRequireDefault(_makeNestHydrationSpec);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = require('debug')('join-monster');

module.exports = (ast, dbCall) => {
  // we need to read the query AST and build a new "SQL AST" from which the SQL and
  // the spec for the Nest Hydration will be determined
  const sqlAST = (0, _queryASTToSqlAST2.default)(ast);
  debug('\nSQL_AST\n', _util2.default.inspect(sqlAST, { depth: 10 }));

  // now convert the "SQL AST" to sql
  const sql = (0, _stringifySqlAST2.default)(sqlAST);
  debug('\nSQL\n', _util2.default.inspect(sql, null, null));

  // and generate the nest hydration spec
  const nestSpec = (0, _makeNestHydrationSpec2.default)(sqlAST);
  debug('\nNEST_SPEC\n', _util2.default.inspect(nestSpec, { depth: 10 }));

  const result = dbCall(sql);
  // if their func gave us a promise for the data, wait for the data
  if (result.then) {
    return result.then(rows => {
      debug('\nRAW DATA\n', _util2.default.inspect(rows, { depth: 10 }));
      return (0, _nesthydrationjs.nest)(rows, nestSpec);
    });
    // otherwise, they were supposed to give us the data
  } else {
    // WARNING: this path is untested
    return Promise.resolve((0, _nesthydrationjs.nest)(result, nestSpec));
  }
};