'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = makeNestHydrationSpec;

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function makeNestHydrationSpec(topNode) {
  validateSqlAST(topNode);
  return _makeNestHydrationSpecForSqlAST(null, '', topNode);
}

function _makeNestHydrationSpecForSqlAST(parent, prefix, node) {
  if (node.table) {
    const prefixToPass = parent ? prefix + node.as + '__' : prefix;
    let columnDeps = new Set();
    for (let c of node.children) {
      var _context;

      if (c.columnDeps) c.columnDeps.forEach((_context = columnDeps).add.bind(_context));
    }

    columnDeps = Array.from(columnDeps);
    const columns = node.children.filter(c => c.column);
    const tables = node.children.filter(c => c.table);

    const oneSpec = _extends({}, _.fromPairs(columnDeps.map(col => [col, prefixToPass + col])), _.fromPairs(columns.map(c => [c.fieldName, prefixToPass + c.column])), _.fromPairs(tables.map(t => [t.fieldName, _makeNestHydrationSpecForSqlAST(node, prefixToPass, t)])));

    if (node.grabMany) {
      return [oneSpec];
    } else {
      return oneSpec;
    }
  } else {
    throw new Error('unexpected/unknown node type reached: ' + _util2.default.inspect(node));
  }
}

function validateSqlAST(topNode) {
  // topNode should not have a sqlJoin entry...
  (0, _assert2.default)(topNode.sqlJoin == null);
}